import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';
import {DayPilot} from 'daypilot-pro-angular';
import {HttpClient} from "@angular/common/http";

@Injectable()
export class DataService {

  // in-memory object that simulates a database backend
  resources: DayPilot.ResourceData[] = [
    { name: "Group 1", id: "G1", expanded: true, children: [
      { name: "Resource 1", id: "R1"},
      { name: "Resource 2", id: "R2"},
    ]},
    { name: "Group 2", id: "G2", expanded: true, children: [
      { name: "Resource 3", id: "R3"},
      { name: "Resource 4", id: "R4"},
    ]},
  ];

  // in-memory object that simulates a database backend
  events: any[] = [
    {
      id: "1",
      resource: "R1",
      start: "2023-01-05",
      end: "2023-01-07",
      text: "Event 1",
      barColor: "#fcb711"
    },
    {
      id: "2",
      resource: "R2",
      start: "2023-01-04",
      end: "2023-01-09",
      text: "Event 2",
      barColor: "#f37021"
    },
    {
      id: "3",
      resource: "R3",
      start: "2023-01-05",
      end: "2023-01-10",
      text: "Event 3",
      barColor: "#cc004c"
    },
    {
      id: "4",
      resource: "R4",
      start: "2023-01-04",
      end: "2023-01-07",
      text: "Event 4",
      barColor: "#0089d0"
    },
  ];

  constructor(private http : HttpClient){
  }

  getEvents(from: DayPilot.Date, to: DayPilot.Date): Observable<any[]> {
    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next(this.events);
      }, 200);
    });

    // return this.http.get("/api/events?from=" + from.toString() + "&to=" + to.toString());
  }

  getResources(): Observable<any[]> {
    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next(this.resources);
      }, 200);
    });

    // return this.http.get("/api/resources");
  }

  createEvent(data: CreateEventParams): Observable<EventData> {
    let e: EventData = {
      start: data.start,
      end: data.end,
      resource: data.resource,
      id: DayPilot.guid(),
      text: data.text
    };

    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next(e);
      }, 200);
    });

    //return this.http.post("/api/events/create", data);
  }

  updateEvent(data: DayPilot.Event): Observable<any> {
    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next({result: "OK"});
      }, 200);
    });
  }

  updateResource(row: DayPilot.Row, name: string): Observable<any> {
    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next({result: "OK"});
        observer.complete();
      }, 200);
    });
  }

  createResource(parent: DayPilot.Row | null, name: string): Observable<ResourceData> {
    let resource: ResourceData = {
      name: name,
      id: DayPilot.guid(),
    };

    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next(resource);
        observer.complete();
      }, 200);
    });
  }

  deleteResource(row: DayPilot.Row): Observable<any> {
    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next({result: "OK"});
        observer.complete();
      }, 200);
    });
  }

  moveResource(source: DayPilot.Row, target: DayPilot.Row, position: string): Observable<any> {
    // simulating an HTTP request
    return new Observable(observer => {
      setTimeout(() => {
        observer.next({result: "OK"});
        observer.complete();
      }, 200);
    });
  }

}


export interface CreateEventParams {
  start: string;
  end: string;
  text: string;
  resource: string | number;
}

export interface UpdateEventParams {
  id: string | number;
  start: string;
  end: string;
  text: string;
  resource: string | number;
}

export interface EventData {
  id: string | number;
  start: string;
  end: string;
  text: string;
  resource: string | number;
}

export interface ResourceData {
  id: string;
  name: string;
}
