import {Component, EventEmitter, Output, ViewChild} from '@angular/core';
import {DayPilot, DayPilotModalComponent} from "daypilot-pro-angular";
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {DataService} from "./data.service";

@Component({
  selector: 'edit-dialog',
  template: `
    <daypilot-modal #modal (close)="closed($event)">
    <div class="center">
      <h1>Edit Resource</h1>
      <form [formGroup]="form">
        <div class="form-item">
          <input formControlName="name" type="text" placeholder="Resource Name"> <span *ngIf="!form.controls.name.valid">Resource name required</span>
        </div>
        <div class="form-item">
          <button (click)="submit()" [disabled]="!form.valid">Save</button>
          <button (click)="cancel()">Cancel</button>
        </div>
    </form>
    </div>
    </daypilot-modal>
  `,
  styles: [`
  .center {
    max-width: 800px;
    margin-left: auto;
    margin-right: auto;
  }
  .form-item {
    margin: 4px 0px;
  }
  `]
})
export class EditComponent {
  @ViewChild("modal") modal : DayPilotModalComponent;
  @Output() close = new EventEmitter();

  form: FormGroup;

  row: DayPilot.Row;

  constructor(private fb: FormBuilder, private ds: DataService) {
    this.form = this.fb.group({
      name: ["", Validators.required],
    });
  }

  show(row: DayPilot.Row) {
    this.row = row;
    this.form.setValue({
      name: row.name,
    });
    this.modal.show();
  }

  submit() {
    let data = this.form.getRawValue();

    this.ds.updateResource(this.row, data.name).subscribe(result => {

      // update the client-side resource object
      this.row.data.name = data.name;

      this.modal.hide(result);
    });
  }

  cancel() {
    this.modal.hide();
  }

  closed(args) {
    this.close.emit(args);
  }

}

