import {AfterViewInit, ChangeDetectorRef, Component, ViewChild} from "@angular/core";
import {DayPilot, DayPilotSchedulerComponent} from "daypilot-pro-angular";
import {DataService} from "./data.service";

@Component({
  selector: 'scheduler-component',
  template: `
    <daypilot-scheduler [config]="config" [events]="events" #scheduler></daypilot-scheduler>
  `,
  styles: [`

  `]
})
export class SchedulerComponent implements AfterViewInit {

  @ViewChild("scheduler") scheduler!: DayPilotSchedulerComponent;

  events: any[] = [];

  menu: DayPilot.Menu = new DayPilot.Menu({
    items: [
      {
        text: "Edit...",
        onClick: async args => {
          const form = [
            {name: "Resource name", id: "name"}
          ];
          const row = args.source;
          const data = row.data;
          const modal = await DayPilot.Modal.form(form, data);
          if (modal.canceled) {
            return;
          }
          const name = modal.result.name;
          this.ds.updateResource(row, name).subscribe(result => {
            row.data.name = name;
          });
        }
      },
      {
        text: "Delete",
        onClick: args => {
          let row = args.source;
          this.ds.deleteResource(row).subscribe(result => {
            row.remove();
            this.scheduler.control.message("Deleted");
          });
        }
      },
      {
        text: "-"
      },
      {
        text: "Add Child...",
        onClick: async args => {
          const form = [
            {name: "Resource name", id: "name"}
          ];
          const row = args.source;
          const data = {};
          const modal = await DayPilot.Modal.form(form, data);
          if (modal.canceled) {
            return;
          }
          const name = modal.result.name;
          this.ds.createResource(row, name).subscribe(resource => {

            // add the new resource to the tree
            let parent = row.data;
            parent.expanded = true;
            if (!parent.children) {
              parent.children = [];
            }
            parent.children.push(resource);
          });
        }
      }
    ]
  });

  config: DayPilot.SchedulerConfig = {
    timeHeaders: [
      {groupBy: "Month", format: "MMMM yyyy"},
      {groupBy: "Day", format: "d"}
    ],
    eventHeight: 40,
    scale: "Day",
    days: 31,
    startDate: "2023-01-01",
    treeEnabled: true,
    rowMoveHandling: "Update",
    durationBarHeight: 5,
    onRowMoved: args => {
      this.ds.moveResource(args.source, args.target, args.position).subscribe(result => this.scheduler.control.message("Moved"));
    },

    rowCreateHandling: "Enabled",
    rowCreateHtml: "New group...",
    onRowCreate: args => {
      // notify the backend; when successful, add the new resource to the config to display it
      this.ds.createResource(null, args.text).subscribe(result => {
        if (!this.config.resources) {
          this.config.resources = [];
        }
        this.config.resources.push(result);
        this.scheduler.control.message("Created.");
      });
    },

    contextMenuResource: this.menu,
    onBeforeRowHeaderRender: args => {
      args.row.areas = [
        {
          right: 2,
          top: 2,
          width: 14,
          height: 14,
          symbol: "assets/daypilot.svg#minichevron-down-2",
          style: "font-size: 12px; background-color: #f9f9f9; border: 1px solid #ccc; padding: 2px; cursor:pointer",
          visibility: "Hover",
          action: "ContextMenu"
        }
      ];
    },
  };

  constructor(private ds: DataService) {
  }

  ngAfterViewInit(): void {
    this.ds.getResources().subscribe(result => this.config.resources = result);

    var from = this.scheduler.control.visibleStart();
    var to = this.scheduler.control.visibleEnd();
    this.ds.getEvents(from, to).subscribe(result => {
      this.events = result;
    });
  }

}

